from sqlalchemy import *
from sqlalchemy.orm import mapper, sessionmaker
import datetime
import os


class ClientDB:
    """
    Класс, определяющий, создающий и изменяющий клиентскую базу данных
    """

    class User:
        def __init__(self, name):
            self.id = None
            self.name = name

        def __repr__(self):
            return self.name

    class Contact:
        def __init__(self, name):
            self.id = None
            self.name = name

        def __repr__(self):
            return self.name

    class MessageHistory:
        def __init__(self, sender, receiver, message):
            self.id = None
            self.sender = sender
            self.receiver = receiver
            self.time = datetime.datetime.now()
            self.message = message

        def __repr__(self):
            return f'From {self.sender} to {self.receiver} at {self.time}: \n {self.message}'

    def __init__(self, client_name):
        self.client_name = client_name
        path = os.path.join(os.path.dirname(os.path.realpath(__file__)), f'client_base_{self.client_name}.db3')
        self.engine = create_engine(f'sqlite:///{path}', echo=False, pool_recycle=7200,
                                    connect_args={'check_same_thread': False})
        self.metadata = MetaData()

        users_table = Table('users', self.metadata,
                            Column('id', Integer, primary_key=True),
                            Column('name', String, unique=True)
                            )

        contacts_table = Table('contacts', self.metadata,
                               Column('id', Integer, primary_key=True),
                               Column('name', String, unique=True)
                               )

        message_history_table = Table('message_history', self.metadata,
                                      Column('id', Integer, primary_key=True),
                                      Column('sender', String),
                                      Column('receiver', String),
                                      Column('time', DateTime),
                                      Column('message', Text)
                                      )

        self.metadata.create_all(self.engine)

        mapper(self.User, users_table)
        mapper(self.Contact, contacts_table)
        mapper(self.MessageHistory, message_history_table)

        self.session = sessionmaker(bind=self.engine)()
        self.session.query(self.Contact).delete()
        self.session.commit()

    def get_users(self):
        """
        Метод, возвращающий список доступных пользователей из базы данных текущего пользователя
        """
        return [user[0] for user in self.session.query(self.User.name).all()]

    def get_contacts(self):
        """
        Метод, возвращающий список контактов текущего пользователя из его базы данных
        """

        return [contact[0] for contact in self.session.query(self.Contact.name).all()]

    def get_message_history(self, contact):
        """
        :param contact: имя пользователя, историю сообщений с которым нужно получить

        Метод, возвращающий историю сообщений текущего пользователя с другим пользователем
        """
        history = self.session.query(self.MessageHistory).filter(or_(self.MessageHistory.sender == contact, self.MessageHistory.receiver == contact))
        return [(message.sender, message.receiver, message.message, message.time)
                for message in history.all()]

    def clear_contacts(self):
        """
        Метод, очищающий таблицу контактов текущего пользователя
        """

        self.session.query(self.Contact).delete()
        self.session.commit()

    def add_contact(self, contact):
        """
        :param contact: имя пользователя, которого нужно добавить в таблицу контактов текущего пользователя

        Метод, добавляющий в таблицу контактов нового пользователя
        """
        if not self.session.query(self.Contact).filter_by(name=contact).count():
            self.session.add(self.Contact(contact))
            self.session.commit()

    def delete_contact(self, contact):
        """
        :param contact: имя пользователя, которого нужно удалить из таблицы контактов текущего пользователя

        Метод, удаляющий из таблицы контактов данного пользователя
        """
        self.session.query(self.Contact).filter_by(name=contact).delete()
        self.session.commit()

    def save_message(self, sender, receiver, message):
        """
        :param sender: отправитель сообщения
        :param receiver: получатель сообщения
        :param message: текст сообщения

        Метод, сохраняющий сообщение в таблице сообщений текущего пользователя
        """
        self.session.add(self.MessageHistory(sender, receiver, message))
        self.session.commit()

    def renew_users(self, users):
        """
        :param users: доступные пользователи, которыми нужно заполнить таблицу

        Метод, обновляющий таблицу доступных пользователей
        """
        self.session.query(self.User).delete()
        for user in users:
            self.session.add(self.User(user))
        self.session.commit()


if __name__ == '__main__':
    test_db = ClientDB('client_1')

    print(' ---- test_db.get_users() ----')
    print(test_db.get_users())

    print(' ---- test_db.renew_users(["client_2", "client_3"]) ----')
    test_db.renew_users(["client_2", "client_3"])
    print(test_db.get_users())

    print(' ---- test_db.get_contacts() ----')
    print(test_db.get_contacts())

    print(' ---- test_db.add_contact("client_2") ----')
    test_db.add_contact("client_2")
    print(test_db.get_contacts())

    print(' ---- test_db.add_contact("client_2") ----')
    test_db.add_contact("client_2")

    print(' ---- test_db.add_contact("client_14") ----')
    test_db.add_contact("client_14")

    print(' ---- test_db.delete_contact("client_2") ----')
    test_db.delete_contact("client_2")
    print(test_db.get_contacts())

    print(' ---- test_db.save_message("client_1", "client_2", "hi") ---- \n'
          ' ---- test_db.get_message_history()                      ---- ')
    test_db.save_message("client_1", "client_2", "hi")
    print(test_db.get_message_history())



import json
import sys
from common.decos import log
from common.variables import MAX_PACKAGE_LENGTH, ENCODING


@log
def get_message(client):
    """
    :param client: сокет с которого должно прийти сообщение
    Функция, осуществляющая приём сообщения с определенного сокета
    """

    encoded_response = client.recv(MAX_PACKAGE_LENGTH)
    json_response = encoded_response.decode(ENCODING)
    response = json.loads(json_response)
    if isinstance(response, dict):
        return response
    else:
        raise TypeError


@log
def send_message(sock, message):
    """
    :param sock: сокет, на который нужно отправить сообщение
    :param message: словарь сообщения для отправки
    Функция, осуществляющая отправку сообщений на определённый сокет
    """

    js_message = json.dumps(message)
    encoded_message = js_message.encode(ENCODING)
    sock.send(encoded_message)



import logging
from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton, QApplication
from PyQt5.QtCore import Qt
import sys

logger = logging.getLogger('client_dist')


class AddContactDialog(QDialog):
    """
    Класс графического интерфейса окна добавления одного пользователя в список контактов текущего пользователя
    """

    def __init__(self, transport, db):
        super().__init__()
        self.transport = transport
        self.db = db

        self.setWindowTitle('Выберите контакт для добавления')
        self.setFixedSize(350, 120)
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setModal(True)

        self.selector_label = QLabel('Выберите контакт для добавления: ', self)
        self.selector_label.setFixedSize(200, 20)
        self.selector_label.move(10, 0)

        self.selector = QComboBox(self)
        self.selector.setFixedSize(200, 20)
        self.selector.move(10, 30)

        self.refresh_button = QPushButton('Обновить список', self)
        self.refresh_button.setFixedSize(100, 30)
        self.refresh_button.move(60, 60)

        self.ok_button = QPushButton('Добавить', self)
        self.ok_button.setFixedSize(100, 30)
        self.ok_button.move(230, 20)

        self.cancel_button = QPushButton('Отмена', self)
        self.cancel_button.setFixedSize(100, 30)
        self.cancel_button.move(230, 60)
        self.cancel_button.clicked.connect(self.close)

        self.get_possible_contacts()
        self.refresh_button.clicked.connect(self.renew_possible_contacts)

    def get_possible_contacts(self):
        """
        Метод, создающий список пользователей, доступных для добавления в список контактов текущего пользователя
        """
        self.selector.clear()
        contacts = set(self.db.get_contacts())
        users = set(self.db.get_users())
        users.remove(self.transport.name)
        self.selector.addItems(users - contacts)

    def renew_possible_contacts(self):
        """
        Метод, обеспечивающий обновление информации в списке пользователей, доступных для добавления в список
        контактов текущего пользователя
        """

        try:
            self.transport.renew_users()
        except OSError:
            pass
        else:
            logger.debug('Обновление списка пользователей с сервера выполнено')
            self.get_possible_contacts()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    from client_db import ClientDB
    db = ClientDB('test1')
    from transport import ClientTransport
    transport = ClientTransport(7777, '127.0.0.1', db, 'test1')
    window = AddContactDialog(transport, db)
    window.show()
    app.exec_()



from PyQt5.QtWidgets import QMainWindow, qApp, QMessageBox, QApplication
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QBrush, QColor
from PyQt5.QtCore import pyqtSlot, Qt
import sys
import json
from Cryptodome.Cipher import PKCS1_OAEP
from Cryptodome.PublicKey import RSA
import base64
from client_files.main_window_ui import MainClientWindowUI
from client_files.add_contact_dialog import AddContactDialog
from client_files.delete_contact_dialog import DeleteContactDialog
from common.errors import ServerError
from common.variables import *

logger = logging.getLogger('client_dist')


class ClientMainWindow(QMainWindow):
    """
    Класс графического интерфейса главного окна клиента пользователя
    """

    def __init__(self, transport, db, keys):
        super().__init__()
        self.transport = transport
        self.db = db
        self.decrypter = PKCS1_OAEP.new(keys)

        self.ui = MainClientWindowUI(self)

        self.ui.menu_exit.triggered.connect(qApp.exit)
        self.ui.send_button.clicked.connect(self.send_message)

        self.ui.add_contact_button.clicked.connect(self.add_contact_dialog)
        self.ui.menu_add_contact.triggered.connect(self.add_contact_dialog)

        self.ui.remove_contact_button.clicked.connect(self.delete_contact_dialog)
        self.ui.menu_del_contact.triggered.connect(self.delete_contact_dialog)

        self.contacts_model = None
        self.history_model = None
        self.messages = QMessageBox()
        self.current_chat = None
        self.current_chat_key = None
        self.encryptor = None
        self.ui.messages.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.ui.messages.setWordWrap(True)

        self.ui.contacts.doubleClicked.connect(self.select_active_user)

        self.renew_clients()
        self.set_disabled_input()
        self.show()

    def set_disabled_input(self):
        """
        Метод, отключающий возможность набора и отправки сообщений для текущего пользователя
        """

        self.ui.label_new_message.setText('Для выбора получателя '
                                          'дважды кликните на нем в окне контактов.')
        self.ui.text_message.clear()
        if self.history_model:
            self.history_model.clear()

        self.ui.clear_button.setDisabled(True)
        self.ui.send_button.setDisabled(True)
        self.ui.text_message.setDisabled(True)

        self.encryptor = None
        self.current_chat = None
        self.current_chat_key = None

    def renew_message_history(self):
        """
        Метод, обновляющий историю сообщений текущего пользователя
        """

        list_messages = sorted(self.db.get_message_history(self.current_chat),
                               key=lambda m: m[3])
        if not self.history_model:
            self.history_model = QStandardItemModel()
            self.ui.messages.setModel(self.history_model)
        self.history_model.clear()
        length = len(list_messages)
        start_index = 0
        if length > 20:
            start_index = length - 20
        for i in range(start_index, length):
            item = list_messages[i]
            if item[1] == self.transport.name:
                mess = QStandardItem(f'Входящее от {item[3].replace(microsecond=0)}:\n {item[2]}')
                mess.setEditable(False)
                mess.setBackground(QBrush(QColor(255, 213, 213)))
                mess.setTextAlignment(Qt.AlignLeft)
                self.history_model.appendRow(mess)
            else:
                mess = QStandardItem(f'Исходящее от {item[3].replace(microsecond=0)}:\n {item[2]}')
                mess.setEditable(False)
                mess.setTextAlignment(Qt.AlignRight)
                mess.setBackground(QBrush(QColor(204, 255, 204)))
                self.history_model.appendRow(mess)
        self.ui.messages.scrollToBottom()

    def select_active_user(self):
        """
        Метод, получающий информацию о новом активном чате текущего пользователя
        """

        self.current_chat = self.ui.contacts.currentIndex().data()
        self.set_active_user()

    def set_active_user(self):
        """
        Метод, устанавливающий новый активный чат для текущего пользователя
        """

        try:
            self.current_chat_key = self.transport.get_key(self.current_chat)
            logger.debug(f'Загружен открытый ключ для {self.current_chat}')
            if self.current_chat_key:
                self.encryptor = PKCS1_OAEP.new(RSA.import_key(self.current_chat_key))
        except (OSError, json.JSONDecodeError):
            self.current_chat_key = None
            self.encryptor = None
            logger.debug(f'Не удалось получить ключ для {self.current_chat}')

        if not self.current_chat_key:
            self.messages.warning(
                self, 'Ошибка', 'Для выбранного пользователя нет ключа шифрования.')
            return

        self.ui.label_new_message.setText(
            f'Введите сообщенние для {self.current_chat}:')
        self.ui.clear_button.setDisabled(False)
        self.ui.send_button.setDisabled(False)
        self.ui.text_message.setDisabled(False)

        self.renew_message_history()

    def renew_clients(self):
        """
        Метод, обновляющий список контактов текущего пользователя
        """

        contacts_list = self.db.get_contacts()
        self.contacts_model = QStandardItemModel()
        for i in sorted(contacts_list):
            item = QStandardItem(i)
            item.setEditable(False)
            self.contacts_model.appendRow(item)
        self.ui.contacts.setModel(self.contacts_model)

    def add_contact_dialog(self):
        """
        Метод, вызывающий окно добавления нового контакта для текущего пользователя
        """

        global select_dialog
        select_dialog = AddContactDialog(self.transport, self.db)
        select_dialog.ok_button.clicked.connect(lambda: self.add_contact_action(select_dialog))
        select_dialog.show()

    def add_contact_action(self, item):
        """
        :param item: окно добавления нового контакта для текущего пользователя
        Метод, провоцирующий добавление нового контакта и закрывающий его окно
        """

        new_contact = item.selector.currentText()
        self.add_contact(new_contact)
        item.close()

    def add_contact(self, new_contact):
        """
        :param new_contact: имя пользователя, которого нужно добавить в контакты текущего пользователя
        Метод, провоцирующий добавление нового пользователя в контакты текущего пользователя на уровне сервера,
        клиентской базы данных и окна графического интерфейса текущего пользователя
        """

        try:
            self.transport.add_contact(new_contact)
        except ServerError as err:
            self.messages.critical(self, 'Ошибка сервера', err.text)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером!')
                self.close()
            self.messages.critical(self, 'Ошибка', 'Таймаут соединения!')
        else:
            self.db.add_contact(new_contact)
            new_contact = QStandardItem(new_contact)
            new_contact.setEditable(False)
            self.contacts_model.appendRow(new_contact)
            logger.info(f'Успешно добавлен контакт {new_contact}')
            self.messages.information(self, 'Успех', 'Контакт успешно добавлен.')

    def delete_contact_dialog(self):
        """
        Метод, вызывающий окно удаления пользователя из контактов текущего пользователя
        """

        global remove_dialog
        remove_dialog = DeleteContactDialog(self.db)
        remove_dialog.ok_button.clicked.connect(lambda: self.delete_contact(remove_dialog))
        remove_dialog.show()

    def delete_contact(self, item):
        """
        :param item: окно удаления пользователя из контактов текущего пользователя
        Метод, провоцирующий удаление данного пользователя из списка контактов текущего пользователя на уровне сервера,
        клиентской базы данных и окна графического интерфеса текущего пользователя
        """

        selected = item.selector.currentText()
        try:
            self.transport.delete_contact(selected)
        except ServerError as err:
            self.messages.critical(self, 'Ошибка сервера', err.text)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером!')
                self.close()
            self.messages.critical(self, 'Ошибка', 'Таймаут соединения!')
        else:
            self.db.delete_contact(selected)
            self.renew_clients()
            logger.info(f'Успешно удалён контакт {selected}')
            self.messages.information(self, 'Успех', 'Контакт успешно удалён.')
            item.close()
            if selected == self.current_chat:
                self.current_chat = None
                self.set_disabled_input()

    def send_message(self):
        """
        Метод, осуществляющий отправку сообщения от текущего пользователя другому пользователю, а также провоцирующее
        сохранение этого сообщения в базе данных текущего пользователя и обновление истроии сообщений между текущим
        и другим пользователями
        """
        message_text = self.ui.text_message.toPlainText()
        self.ui.text_message.clear()
        if not message_text:
            return
        message_text_encrypted = base64.b64encode(self.encryptor.encrypt(message_text.encode('utf8')))
        try:
            self.transport.send_message(self.current_chat, message_text_encrypted.decode('ascii'))
            pass
        except ServerError as err:
            self.messages.critical(self, 'Ошибка', err.text)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером!')
                self.close()
            self.messages.critical(self, 'Ошибка', 'Таймаут соединения!')
        except (ConnectionResetError, ConnectionAbortedError):
            self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером!')
            self.close()
        else:
            self.db.save_message(self.transport.name, self.current_chat, message_text)
            logger.debug(f'Отправлено сообщение для {self.current_chat}: {message_text}')
            self.renew_message_history()

    @pyqtSlot(dict)
    def new_message_slot(self, message):
        """
        :param message: Словарь, содержащий данные о сообщении
        Слот, отвечающий на сигнал получения нового сообщения текущим пользователем. Декодирует сообщение, провоцирует
        его сохранение в базе данных текущего пользователя и оповещает его об этом путём создания соответствующего
        окна
        """

        encrypted_message = base64.b64decode(message[MESSAGE_TEXT])
        try:
            decrypted_message = self.decrypter.decrypt(encrypted_message)
        except (ValueError, TypeError):
            self.messages.warning(self, 'Ошибка', 'Не удалось декодировать сообщение.')
            return
        self.db.save_message(
            message[SENDER],
            self.transport.name,
            decrypted_message.decode('utf8'))
        sender = message[SENDER]
        if sender == self.current_chat:
            self.renew_message_history()
        else:
            if sender in self.db.get_contacts():
                if self.messages.question(self, 'Новое сообщение',
                                          f'Получено новое сообщение от {sender}, '
                                          f'открыть чат с ним?', QMessageBox.Yes,
                                          QMessageBox.No) == QMessageBox.Yes:
                    self.current_chat = sender
                    self.set_active_user()
            else:
                print('NO')
                if self.messages.question(self, 'Новое сообщение',
                                          f'Получено новое сообщение от {sender}.\n '
                                          f'Данного пользователя нет в вашем контакт-листе.\n'
                                          f' Добавить в контакты и открыть чат с ним?',
                                          QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes:
                    self.add_contact(sender)
                    self.current_chat = sender
                    self.set_active_user()

    @pyqtSlot()
    def connection_lost_slot(self):
        """
        Слот, отвечающий на сигнал о потере соединения с сервером и создающий соответствующее окно, оповещающее текущего
        пользователя об этом
        """
        self.messages.warning(self, 'Сбой соединения', 'Потеряно соединение с сервером. ')
        self.close()

    @pyqtSlot()
    def signal_205(self):
        """
        Слот, отвечающий на сигнал об изменении в таблице пользователей серверной базы данных, и провоцирующий
        обновлении списка контактов текущего клиента
        """

        if self.current_chat and self.current_chat not in self.db.get_users():
            self.messages.warning(
                self,
                'Сочувствую',
                'К сожалению собеседник был удалён с сервера.')
            self.set_disabled_input()
            self.current_chat = None
        self.renew_clients()

    def make_connection(self, transport):
        """
        :param transport: сокет клиента
        Метод, соединяющий сигналы со слотами
        """
        transport.new_message.connect(self.new_message_slot)
        transport.connection_lost.connect(self.connection_lost_slot)
        transport.message_205.connect(self.signal_205)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    from client_db import ClientDB
    db = ClientDB('test1')
    from transport import ClientTransport
    transport = ClientTransport(7777, '127.0.0.1', 'test1', db)
    window = ClientMainWindow(transport, db)
    sys.exit(app.exec_())




import socket
import sys
import time
import threading
from PyQt5.QtCore import pyqtSignal, QObject
import hashlib
import binascii
import hmac
from common.utils import *
from common.variables import *
from common.errors import ServerError


logger = logging.getLogger('client_dist')

transport_lock = threading.Lock()


class ClientTransport(threading.Thread, QObject):
    """
    Класс, осуществляющий функционал клиента
    """

    new_message = pyqtSignal(dict)
    connection_lost = pyqtSignal()
    message_205 = pyqtSignal()

    def __init__(self, port, ip, name, db, password, keys):
        """
        Метод __init__, который помимо стандартных своих функций проверяет пароль пользователя и устанавливает
        соединение с сервером
        """
        threading.Thread.__init__(self)
        QObject.__init__(self)
        self.daemon = True
        self.name = name
        self.db = db
        self.password = password
        self.keys = keys

        self.transport = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.transport.settimeout(5)

        connected = False
        for i in range(5):
            logger.info(f'Попытка подключения №{i + 1}')
            try:
                self.transport.connect((ip, port))
            except (OSError, ConnectionRefusedError):
                pass
            else:
                connected = True
                logger.debug('Установлено соединение с сервером')
                break
            time.sleep(1)

        if not connected:
            logger.critical('Не удалось установить соединение с сервером')
            raise ServerError('Не удалось установить соединение с сервером')

        password_bytes = self.password.encode('utf-8')
        salt = self.name.lower().encode('utf-8')
        password_hash = binascii.hexlify(hashlib.pbkdf2_hmac('sha512', password_bytes, salt, 10000))
        logger.debug(f'Password hash ready: {password_hash}')

        pubkey = self.keys.publickey().export_key().decode('ascii')

        with transport_lock:
            presence = {
                ACTION: PRESENCE,
                TIME: time.time(),
                USER: {
                    ACCOUNT_NAME: self.name,
                    PUBLIC_KEY: pubkey
                }
            }
            logger.debug(f"Presense message = {presence}")
            try:
                send_message(self.transport, presence)
                ans = get_message(self.transport)
                logger.debug(f'Server response = {ans}.')
                if RESPONSE in ans:
                    if ans[RESPONSE] == 400:
                        raise ServerError(ans[ERROR])
                    elif ans[RESPONSE] == 511:
                        ans_data = ans[BIN]
                        hash = hmac.new(password_hash, ans_data.encode('utf-8'), 'MD5').digest()
                        my_ans = RESPONSE_511
                        my_ans[BIN] = binascii.b2a_base64(hash).decode('ascii')
                        send_message(self.transport, my_ans)
                        self.process_response_ans(get_message(self.transport))
            except (OSError, json.JSONDecodeError) as err:
                logger.debug(f'Connection error.', exc_info=err)
                raise ServerError('Сбой соединения в процессе авторизации.')

        try:
            self.renew_users()
            self.renew_contacts()
        except OSError as err:
            if err.errno:
                logger.critical(f'Потеряно соединение с сервером.')
                raise ServerError('Потеряно соединение с сервером!')
            logger.error('Timeout соединения при обновлении списков пользователей.')
        except json.JSONDecodeError:
            logger.critical(f'Потеряно соединение с сервером.')
            raise ServerError('Потеряно соединение с сервером!')

        self.running = True

    def renew_users(self):
        """
        Метод, обновляющий таблицу доступных пользователей в базе данных текущего пользователя
        """

        with transport_lock:
            send_message(self.transport, {
                ACTION: GET_USERS,
                TIME: time.time(),
                ACCOUNT_NAME: self.name
            })
            response = get_message(self.transport)
        logger.info(f'Получено сообщение от сервера {response}')
        if RESPONSE in response and response[RESPONSE] == 202 and DATA in response and isinstance(response[DATA],
                                                                                                  list):
            self.db.renew_users(response[DATA])
        else:
            logger.error('Не удалось обновить список доступных пользователей')

    def renew_contacts(self):
        """
        Метод, обновляющий таблицу контактов в базе данных текущего пользователя
        """

        with transport_lock:
            send_message(self.transport, {
                ACTION: GET_CONTACTS,
                TIME: time.time(),
                ACCOUNT_NAME: self.name
            })
            response = get_message(self.transport)
        logger.info(f'Получено сообщение от сервера {response}')
        if RESPONSE in response and response[RESPONSE] == 202 and DATA in response and isinstance(response[DATA],
                                                                                                  list):
            for contact in response[DATA]:
                self.db.add_contact(contact)
        else:
            logger.error('Упс. Что-то пошло не так')

    def get_key(self, user):
        """
        :param user: имя пользователя, чей ключ нужно получить
        Метод, запрашивающий публичный ключ данного пользователя
        """

        logger.debug(f'Запрос публичного ключа для {user}')
        req = {
            ACTION: GET_PUBLIC_KEY,
            TIME: time.time(),
            ACCOUNT_NAME: user
        }
        with transport_lock:
            send_message(self.transport, req)
            response = get_message(self.transport)
        if RESPONSE in response and response[RESPONSE] == 511:
            return response[BIN]
        else:
            logger.error(f'Не удалось получить ключ собеседника{user}.')

    def process_response_ans(self, message):
        """
        :param message: словарь, содержащий данный о полученном сообщении
        Метод, обрабатывающий полученное сообщение
        """

        logger.debug(f'Разбор сообщения: {message}')
        if RESPONSE in message:
            if message[RESPONSE] == 200:
                return
            elif message[RESPONSE] == 400:
                raise ServerError(f'400 : {message[ERROR]}')
            elif message[RESPONSE] == 205:
                self.renew_users()
                self.renew_contacts()
                self.message_205.emit()
            else:
                logger.debug(f'Принят неизвестный код подтверждения {message[RESPONSE]}')
        elif ACTION in message and message[ACTION] == MESSAGE and \
                SENDER in message and MESSAGE_TEXT in message and RECEIVER in message \
                and message[RECEIVER] == self.name:
            logger.info(f'Получено сообщение от пользователя {message[SENDER]}:\n{message[MESSAGE_TEXT]}')
            self.new_message.emit(message)

    def add_contact(self, contact):
        """
        :param contact: имя пользователя, которого нужно добавил в контакты
        Метод, запрашивающий добавление другого пользователя в контакты текущего пользователя
        """

        with transport_lock:
            send_message(self.transport, {ACTION: ADD_CONTACT,
                                          ACCOUNT_NAME: self.name,
                                          TIME: time.time(),
                                          CONTACT: contact
                                          })
            self.process_response_ans(get_message(self.transport))

    def delete_contact(self, contact):
        """
        :param contact: имя пользователя, которого нужно удалить из контактов текущего пользователя
        Метод, запрашивающий удаление другого пользователя из списка текущего пользователя
        """

        with transport_lock:
            send_message(self.transport, {ACTION: DEL_CONTACT,
                                          ACCOUNT_NAME: self.name,
                                          TIME: time.time(),
                                          CONTACT: contact
                                          })
            self.process_response_ans(get_message(self.transport))

    def transport_shutdown(self):
        """
        Метод, осуществляющий завершение работы клиента
        """

        self.running = False
        with transport_lock:
            try:
                send_message(self.transport, {
                        ACTION: EXIT,
                        TIME: time.time(),
                        ACCOUNT_NAME: self.name
                    })
            except OSError:
                pass
        logger.info('Завершение работы')
        time.sleep(0.5)

    def send_message(self, receiver, message):
        """
        :param receiver: имя получателя сообщения
        :param message: текст сообщения
        Метод, запрашивающий отправление сообщения от текущего пользователя другому пользователю
        """

        message_dict = {
            ACTION: MESSAGE,
            SENDER: self.name,
            RECEIVER: receiver,
            TIME: time.time(),
            MESSAGE_TEXT: message
        }
        logger.debug(f'Сформирован словарь сообщения: {message_dict}')

        with transport_lock:
            send_message(self.transport, message_dict)
            self.process_response_ans(get_message(self.transport))
            logger.info(f'Отправлено сообщение для пользователя {receiver}')

    def run(self):
        """
        Метод, отслеживающий входящие сообщения и осуществляющий их приём
        """

        logger.debug('Запущен процесс - приёмник сообщений с сервера.')
        while self.running:
            time.sleep(1)
            message = None
            with transport_lock:
                try:
                    self.transport.settimeout(0.5)
                    message = get_message(self.transport)
                except OSError as err:
                    if err.errno:
                        logger.critical(f'Потеряно соединение с сервером.')
                        self.running = False
                        self.connection_lost.emit()
                except (ConnectionError, ConnectionAbortedError,
                        ConnectionResetError, json.JSONDecodeError, TypeError):
                    logger.debug(f'Потеряно соединение с сервером.')
                    self.running = False
                    self.connection_lost.emit()
                finally:
                    self.transport.settimeout(5)

            if message:
                logger.debug(f'Принято сообщение с сервера: {message}')
                self.process_response_ans(message)


from setuptools import setup, find_packages

setup(name="chargaka_messenger_client",
      version="0.1.1",
      description="chargaka_messenger_client",
      author="Artem",
      author_email="warload22@gmail.com",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )



import sys
import logging
from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton, QApplication
from PyQt5.QtCore import Qt


logger = logging.getLogger('client_dist')


class DeleteContactDialog(QDialog):
    """
    Класс графического интерфейса окна удаления одного пользователя из списка контактов другого пользователя
    """
    def __init__(self, db):
        super().__init__()
        self.db = db

        self.setWindowTitle('Выберите контакт для удаления')
        self.setFixedSize(350, 120)
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setModal(True)

        self.selector_label = QLabel('Выберите контакт для удаления: ', self)
        self.selector_label.setFixedSize(200, 20)
        self.selector_label.move(10, 0)

        self.selector = QComboBox(self)
        self.selector.setFixedSize(200, 20)
        self.selector.move(10, 30)
        self.selector.addItems(sorted(self.db.get_contacts()))

        self.ok_button = QPushButton('Удалить', self)
        self.ok_button.setFixedSize(100, 30)
        self.ok_button.move(230, 20)

        self.cancel_button = QPushButton('Отмена', self)
        self.cancel_button.setFixedSize(100, 30)
        self.cancel_button.move(230, 60)
        self.cancel_button.clicked.connect(self.close)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    from client_db import ClientDB
    db = ClientDB('test1')
    window = DeleteContactDialog(db)
    db.add_contact('test1')
    db.add_contact('test2')
    print(db.get_contacts())
    window.selector.addItems(sorted(db.get_contacts()))
    window.show()
    app.exec_()
